﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Web.UI;
using PdfSharp;
using PdfSharp.Pdf;
using ThinkGeo.MapSuite.Core;

namespace CSSamples.Samples.Extending_MapSuite
{
    public partial class UsePdfExtension : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                Map1.MapUnit = GeographyUnit.DecimalDegree;
                Map1.CurrentExtent = new RectangleShape(-111.7875, 92.0859375, 148.36875, -93.5390625);
                Map1.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean);

                EcwRasterLayer worldImageLayer = new EcwRasterLayer(MapPath("~/SampleData/world/World.ecw"));

                ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(MapPath("~/SampleData/world/cntry02.shp"));
                worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
                worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen.StartCap = DrawingLineCap.Round;
                worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen.EndCap = DrawingLineCap.Round;
                worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                ShapeFileFeatureLayer usStatesLayer = new ShapeFileFeatureLayer(MapPath("~/SampleData/USA/STATES.SHP"));
                usStatesLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.State2;
                usStatesLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen.StartCap = DrawingLineCap.Round;
                usStatesLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                ShapeFileFeatureLayer worldCapitalsLayer = new ShapeFileFeatureLayer(MapPath("~/SampleData/world/capital.shp"));
                worldCapitalsLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.City4;
                worldCapitalsLayer.DrawingMarginPercentage = 80;
                worldCapitalsLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                ShapeFileFeatureLayer worldCapitalsLabelsLayer = new ShapeFileFeatureLayer(MapPath("~/SampleData/world/capital.shp"));
                worldCapitalsLabelsLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.Capital1("city_name");
                worldCapitalsLabelsLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.HaloPen = new GeoPen(GeoColor.StandardColors.White, 2);
                worldCapitalsLabelsLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.HaloPen.StartCap = DrawingLineCap.Round;
                worldCapitalsLabelsLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.HaloPen.EndCap = DrawingLineCap.Round;
                worldCapitalsLabelsLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.SuppressPartialLabels = true;
                worldCapitalsLabelsLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                Map1.StaticOverlay.Layers.Add("WorldImageLayer", worldImageLayer);
                Map1.StaticOverlay.Layers.Add("WorldLayer", worldLayer);
                Map1.StaticOverlay.Layers.Add("USStatesLayer", usStatesLayer);
                Map1.StaticOverlay.Layers.Add("WorldCapitals", worldCapitalsLayer);
                Map1.StaticOverlay.Layers.Add("WorldCapitalsLabels", worldCapitalsLabelsLayer);
            }
        }

        protected void btnToPdf_Click(object sender, EventArgs e)
        {
            PdfDocument document = new PdfDocument();
            PdfPage page = document.AddPage();
            if (rdlOptions.SelectedItem.Text == "Landscape")
            {
                page.Orientation = PageOrientation.Landscape;
            }
            PdfGeoCanvas pdfGeoCanvas = new PdfGeoCanvas();

            // This allows you to control the area in which you want the
            // map to draw in.  Leaving this commented out uses the whole page
            //pdfGeoCanvas.DrawingArea = new Rectangle(200, 50, 400, 400);
            Collection<SimpleCandidate> labelsInLayers = new Collection<SimpleCandidate>();
            foreach (Layer layer in Map1.StaticOverlay.Layers)
            {
                RectangleShape printExtent = ExtentHelper.GetDrawingExtent(Map1.CurrentExtent, (float)Map1.WidthInPixels, (float)Map1.HeightInPixels);
                pdfGeoCanvas.BeginDrawing(page, printExtent, GeographyUnit.DecimalDegree);
                layer.Open();
                layer.Draw(pdfGeoCanvas, labelsInLayers);
                layer.Close();
                pdfGeoCanvas.EndDrawing();
            }

            string filename = Server.MapPath(".") + "\\MapSuite PDF Map.pdf";
            document.Save(filename);
            OpenPdfFile(filename);
        }

        private void OpenPdfFile(string filename)
        {
            if (File.Exists(filename))
            {
                ClientScript.RegisterStartupScript(this.GetType(), "showPdfFile", "<script language='javascript' type='text/javascript'>function A() {window.open('MapSuite PDF Map.pdf');};A();</script>");
            }
            else
            {
                ClientScript.RegisterStartupScript(this.GetType(), "showPdfFile", "<script language='javascript' type='text/javascript'>function A() {alert('No application is associated with the specified file for this operation.');};A();</script>");
            }
        }
    }
}
